@@ -1,6 +1,12 @@
 import importlib.metadata
 
-from akerbp.mlpet import feature_engineering, imputers, preprocessors, utilities
+from akerbp.mlpet import (
+    feature_engineering,
+    imputers,
+    petrophysical_features,
+    preprocessors,
+    utilities,
+)
 from akerbp.mlpet.dataset import Dataset
 from akerbp.mlpet.transformer import MLPetTransformer
 
@@ -12,5 +18,6 @@ __all__ = [
     "imputers",
     "utilities",
     "preprocessors",
+    "petrophysical_features",
     "MLPetTransformer",
 ]
