@@ -2,7 +2,7 @@ import os
 import warnings
 from copy import deepcopy
 from pathlib import Path
-from typing import Any, Dict, List, Set, Union
+from typing import Any, Dict, List, Optional, Set, Union
 
 import numpy as np
 import pandas as pd
@@ -10,11 +10,15 @@ import yaml
 from pandas.core.frame import DataFrame
 from tqdm.auto import tqdm
 
-from akerbp.mlpet import feature_engineering, imputers, preprocessors, utilities
-from akerbp.mlpet.dataloader import DataLoader
+import akerbp.mlpet.data.mappings as default_mappings
+import akerbp.mlpet.dataloader as dl
+import akerbp.mlpet.feature_engineering as feature_engineering
+import akerbp.mlpet.imputers as imputers
+import akerbp.mlpet.preprocessors as preprocessors
+import akerbp.mlpet.utilities as utilities
 
 
-class Dataset(DataLoader):
+class Dataset(dl.DataLoader):
     """
     The main class representing a dataset
 
@@ -30,8 +34,6 @@ class Dataset(DataLoader):
     Note: The id_column is always considered a categorical variable!
 
     Args:
-        mappings: dict or path to a yaml file. If a path is provided it must
-            be provided as an absolute path
         settings: dict or path to a yaml file. If a path is provided it must
             be provided as an absolute path. The possible keys for the settings:
 
@@ -52,6 +54,9 @@ class Dataset(DataLoader):
         folder_path: The path to where preprocessing artifacts are stored/shall
             be saved to. Similar to the other two arguments this path must be
             provided as an absolute path.
+        mappings: (optional) dict or path to a yaml file. If a path is provided it must
+            be provided as an absolute path. Any provided mappings will override
+            the internal mappings in MLPet on a key-by-key basis
 
     """
 
@@ -231,9 +236,9 @@ class Dataset(DataLoader):
 
     def __init__(
         self,
-        mappings: Union[str, Dict[str, str]],
         settings: Union[str, Dict[str, Any]],
         folder_path: Union[str, Path],
+        mappings: Optional[Union[str, Dict[str, str]]] = None,
     ) -> None:
         # Define supported preprocessing functions
         self.supported_preprocessing_functions = {
@@ -267,13 +272,41 @@ class Dataset(DataLoader):
         for key, val in self.settings.items():
             setattr(self, key, val)
 
-        self.__ingest_init_input(att_name="mappings", att_val=mappings)
-        if "curve_mappings" in self.mappings:
-            self.curve_mappings = self.mappings["curve_mappings"]
-        if "formations_map" in self.mappings:
-            self.formations_map = self.mappings["formations_map"]
-        if "groups_map" in self.mappings:
-            self.groups_map = self.mappings["groups_map"]
+        if mappings is None:
+            self.mappings = default_mappings.base_mappings
+        else:
+            self.__ingest_init_input(att_name="mappings", att_val=mappings)
+            # Any keys in the provided mappings override the base mappings
+            try:
+                curve_mappings = default_mappings.base_mappings["curve_mappings"]
+                curve_mappings.update(self.mappings["curve_mappings"])
+                self.mappings["curve_mappings"] = curve_mappings
+            except KeyError:
+                self.mappings["curve_mappings"] = default_mappings.base_mappings[
+                    "curve_mappings"
+                ]
+                pass
+            try:
+                formations_map = default_mappings.base_mappings["formations_map"]
+                formations_map.update(self.mappings["formations_map"])
+                self.mappings["formations_map"] = formations_map
+            except KeyError:
+                self.mappings["formations_map"] = default_mappings.base_mappings[
+                    "formations_map"
+                ]
+                pass
+            try:
+                groups_map = default_mappings.base_mappings["groups_map"]
+                groups_map.update(self.mappings["groups_map"])
+                self.mappings["groups_map"] = groups_map
+            except KeyError:
+                self.mappings["groups_map"] = default_mappings.base_mappings[
+                    "groups_map"
+                ]
+                pass
+        self.curve_mappings = self.mappings["curve_mappings"]
+        self.formations_map = self.mappings["formations_map"]
+        self.groups_map = self.mappings["groups_map"]
 
         # Ensure required settings were provided to prevent problems later down the line
         required = ["id_column"]
@@ -362,7 +395,7 @@ class Dataset(DataLoader):
         # If no dataframe is provided, use class df_original
         if df is None:
             if hasattr(self, "df_original"):
-                df = self.df_original
+                df = self.df_original.copy()
                 if df.empty:
                     raise ValueError(
                         "The class connected pd.Dataframe ('df_original') has "
