@@ -28,7 +28,9 @@ import pandas as pd
 import sklearn.preprocessing
 from sklearn.base import BaseEstimator
 
-from akerbp.mlpet import feature_engineering, imputers, utilities
+import akerbp.mlpet.feature_engineering as feature_engineering
+import akerbp.mlpet.imputers as imputers
+import akerbp.mlpet.utilities as utilities
 
 
 def set_as_nan(df: pd.DataFrame, **kwargs) -> pd.DataFrame:
@@ -141,10 +143,12 @@ def remove_outliers(df: pd.DataFrame, **kwargs) -> pd.DataFrame:
 
             if "NEU" in outlier_curves:
                 outliers = df[(df.NEU < -0.5) | (df.NEU > 1)]
-                if len(outliers) / len_df > th:
+                ratio = len(outliers) / len_df
+                if ratio > th:
                     warnings.warn(
-                        f"NEU has more than {th*100}% of its values higher than 1"
-                        " or lower than -0.5"
+                        f"NEU has {ratio*100}% of its values higher than 1"
+                        " or lower than -0.5. The threshold for this warning is"
+                        f" {th*100}%"
                     )
                 df.loc[df.NEU > 1, "NEU"] = np.nan
                 df.loc[df.NEU < -0.5, "NEU"] = np.nan
