@@ -1071,3 +1071,51 @@ VERTICAL_DF = pd.DataFrame(
         },
     }
 )
+
+DEPTH_TREND_X = pd.DataFrame(
+    {
+        "TVDBML": {
+            0: 1900.7523452000005,
+            1: 1900.90531154,
+            2: 1901.0572781,
+            3: 1901.20924466,
+            4: 1901.362211,
+        },
+        "GROUP": {
+            0: "SHETLAND",
+            1: "SHETLAND",
+            2: "SHETLAND",
+            3: "SHETLAND",
+            4: "SHETLAND",
+        },
+        "FORMATION": {
+            0: "SPRINGAR FM",
+            1: "BRYGGE",
+            2: "SPRINGAR FM",
+            3: "SPRINGAR FM",
+            4: "SPRINGAR FM",
+        },
+        "well": {
+            0: "6201_11-3_R",
+            1: "6201_11-3_R",
+            2: "6201_11-3_R",
+            3: "6201_11-3_R",
+            4: "6201_11-3_R",
+        },
+    }
+)
+
+DEPTH_TREND_Y = pd.DataFrame(
+    {
+        "DEN_trend": {0: 2.263, 1: 2.024, 2: 2.263, 3: 2.263, 4: 2.263},
+        "DEN_trend_P10": {0: 2.111, 1: 1.861, 2: 2.111, 3: 2.111, 4: 2.111},
+        "DEN_trend_P90": {0: 2.415, 1: 2.187, 2: 2.415, 3: 2.415, 4: 2.415},
+        "well": {
+            0: "6201_11-3_R",
+            1: "6201_11-3_R",
+            2: "6201_11-3_R",
+            3: "6201_11-3_R",
+            4: "6201_11-3_R",
+        },
+    }
+)
