@@ -1,8 +1,17 @@
+import os
+
 from cognite.client import CogniteClient
 
 from akerbp.mlpet.dataloader import DataLoader
 
-client = CogniteClient(client_name="test", project="akbp-subsurface")
+api_key = os.getenv("COGNITE_API_KEY_PERSONAL")
+if api_key is not None:
+    client = CogniteClient(
+        client_name="test", project="akbp-subsurface", api_key=api_key
+    )
+else:
+    # Assuming COGNITE_API_KEY is set in the environment
+    client = CogniteClient(client_name="test", project="akbp-subsurface")
 
 
 def test_load_from_cdf():
