@@ -71,15 +71,31 @@ def test_dataset_with_empty_settings():
             "Please refer to the docstring."
         ),
     ):
-        _ = Dataset({}, {}, os.path.abspath(os.getcwd()))
+        _ = Dataset({}, os.path.abspath(os.getcwd()))
 
 
 def test_dataset_with_minimal_settings():
     folder_path = os.path.abspath(os.path.join(os.getcwd(), "foobar"))
     ds = Dataset(
-        {"curve_mappings": {"foo": "foobar"}},
-        {"id_column": "foo", "depth_column": "bar"},
-        folder_path,
+        mappings={"curve_mappings": {"foo": "foobar", "AC": "DUMMY"}},
+        settings={"id_column": "foo", "depth_column": "bar"},
+        folder_path=folder_path,
+    )
+    if os.path.exists(folder_path):
+        os.rmdir(folder_path)
+
+    # Make sure curve defined in base mappings was overridden
+    assert ds.curve_mappings["AC"] == "DUMMY"
+
+    assert ds.all_curves == set(["foobar", "bar"])
+
+
+def test_dataset_with_no_mappings_provided():
+    folder_path = os.path.abspath(os.path.join(os.getcwd(), "foobar"))
+    ds = Dataset(
+        settings={"id_column": "foo", "depth_column": "bar"},
+        folder_path=folder_path,
+        mappings=None,
     )
     if os.path.exists(folder_path):
         os.rmdir(folder_path)
