@@ -1,17 +1,23 @@
-from cognite.client import CogniteClient
+import os
+
 import yaml
+from cognite.client import CogniteClient
 
 from akerbp.mlpet import utilities as utils
 from akerbp.mlpet.tests.data.data import (
     FORMATION_TOPS_MAPPER,
-    VERTICAL_DEPTHS_MAPPER,
     TEST_DF,
+    VERTICAL_DEPTHS_MAPPER,
 )
 
-CLIENT = CogniteClient(
-    client_name="test",
-    project="akbp-subsurface",
-)
+api_key = os.getenv("COGNITE_API_KEY_PERSONAL")
+if api_key is not None:
+    CLIENT = CogniteClient(
+        client_name="test", project="akbp-subsurface", api_key=api_key
+    )
+else:
+    # Assuming COGNITE_API_KEY is set in the environment
+    CLIENT = CogniteClient(client_name="test", project="akbp-subsurface")
 WELL_NAMES = ["25/10-10"]
 
 
@@ -33,9 +39,10 @@ def test_remove_wo_label():
 
 def test_standardize_names():
     mapper = yaml.load(
-        open("akerbp/mlpet/tests/data/test_mappings.yaml", "r"), Loader=yaml.SafeLoader
+        open("src/akerbp/mlpet/tests/data/test_mappings.yaml", "r"),
+        Loader=yaml.SafeLoader,
     )
-    names = utils.standardize_names(TEST_DF.columns.tolist(), mapper=mapper)
+    utils.standardize_names(TEST_DF.columns.tolist(), mapper=mapper)
 
 
 def test_standardize_group_formation_name():
