@@ -3,8 +3,8 @@ from typing import Any, Dict
 import pandas as pd
 from sklearn.base import BaseEstimator, TransformerMixin
 
-from akerbp.mlpet import Dataset
-from akerbp.mlpet.utilities import feature_target_split
+import akerbp.mlpet.dataset as dataset
+import akerbp.mlpet.utilities as utilities
 
 
 class FeatureSelector(BaseEstimator, TransformerMixin):
@@ -26,7 +26,7 @@ class FeatureSelector(BaseEstimator, TransformerMixin):
 class MLPetTransformer(BaseEstimator, TransformerMixin):
     def __init__(
         self,
-        ds: Dataset,
+        ds: dataset.Dataset,
         train_kwargs: Dict[str, Dict[str, Any]] = None,
         test_kwargs: Dict[str, Dict[str, Any]] = None,
         verbose=False,
@@ -87,7 +87,7 @@ class MLPetTransformer(BaseEstimator, TransformerMixin):
 
         # Retrieve X
         if self.ds.label_column in df:
-            X, _ = feature_target_split(df, self.ds.label_column)
+            X, _ = utilities.feature_target_split(df, self.ds.label_column)
         else:
             X = df
 
