@@ -1,16 +1,20 @@
 import importlib.resources
+import json
+import os
 import re
 import warnings
 from functools import lru_cache
-from typing import Any, Dict, List, Set, Tuple, Union
+from typing import Any, Callable, Dict, List, Set, Tuple, Union
 
 import numpy as np
+import numpy.typing as npt
 import pandas as pd
-from cognite.client import CogniteClient
 from cognite.client.data_classes import Asset, AssetList
+from cognite.experimental import CogniteClient
 from numpy import float64
 
 import akerbp.mlpet.data
+import akerbp.mlpet.preprocessors as preprocessors
 
 
 def drop_rows_wo_label(df: pd.DataFrame, label_column: str, **kwargs) -> pd.DataFrame:
@@ -587,7 +591,13 @@ def get_well_metadata(  # noqa: C901
 
     # Need to handle WELLBORE_LOCATION_SPATIAL specially
     if "WELLBORE_LOCATION_SPATIAL" in cdf_meta.index:
-        sub = cdf_meta.loc["WELLBORE_LOCATION_SPATIAL"].dropna().apply(eval).copy()
+        sub = (
+            cdf_meta.loc["WELLBORE_LOCATION_SPATIAL"]
+            .replace("null", np.nan)
+            .dropna()
+            .apply(eval)
+            .copy()
+        )
         restructured = pd.json_normalize(sub)[["x", "y"]]
         restructured.index = sub.index
         restructured = restructured.explode("x").explode("y").T
@@ -628,8 +638,8 @@ def get_well_metadata(  # noqa: C901
     return metadata_dict
 
 
-def get_formation_tops(
-    well_names: str,
+def get_formation_tops(  # noqa: C901
+    well_names: List[str],
     client: CogniteClient,
     **kwargs,
 ) -> Dict[str, Dict[str, Any]]:
@@ -644,6 +654,9 @@ def get_formation_tops(
     Keyword Args:
         undefined_name (str): Name for undefined formation/group tops.
             Defaults to 'UNKNOWN'
+        load_from_gis (bool): Indicator for whether try to fetch formation tops from GIS if not found in CDF.
+            This option require access for user and might give errormessage.
+            Defaults to False
 
     NOTE: The formation will be skipped if it's only 1m thick.
             NPD do not provide technial side tracks,
@@ -670,6 +683,7 @@ def get_formation_tops(
             of the corresponding level entry.
     """
     undefined_name: str = kwargs.get("undefined_name", "UNKNOWN")
+    load_from_gis: str = kwargs.get("load_from_gis", False)
 
     formation_tops_mapper = {}
     for well in well_names:
@@ -683,16 +697,61 @@ def get_formation_tops(
             }
         )
         if tops is None or len(tops) == 0:
-            warnings.warn(
-                f"No formation tops information was found for {well}. Skipping it!"
-            )
-            continue
-        rows = tops[0].rows(start=None, end=None).to_pandas()
+            if load_from_gis:
+                try:
+                    try:
+                        import pyodbc
+                    except ImportError as e:
+                        raise ImportError(
+                            "Could not import pyodbc. This is a very windows spesific package\
+                                and macOS/Linus users might need to install additional updates in order to install pyodbc"
+                        ) from e
 
-        rows_groups = rows[rows.Level == "GROUP"].sort_values(["Top_MD", "Base_MD"])
-        rows_formations = rows[rows.Level == "FORMATION"].sort_values(
-            ["Top_MD", "Base_MD"]
-        )
+                    cnxn = pyodbc.connect(
+                        "DRIVER={SQL Server};SERVER=arcgis2.db.pertra.locale\\felles;DATABASE=GISData;Trusted_Connection=yes;"
+                    )
+                    QUERY = f"SELECT Wellbore, Chronostrat, Lithostrat, Top_MD, Base_MD FROM gisdata.Well_tops WHERE Wellbore='{well_name}'"
+                    rows = pd.read_sql(QUERY, cnxn).sort_values(["Top_MD", "Base_MD"])
+
+                    for i, r in rows.iterrows():
+                        rows.loc[i, "Lithostrat"] = standardize_group_formation_name(
+                            r.Lithostrat
+                        )
+                    rows["FORMATION"], rows["GROUP"] = map_formation_and_group(
+                        form_or_group=rows.Lithostrat, MissingValue="UNKNOWN"
+                    )
+
+                    # For formation without a group recording to mapping
+                    # Assume group is similar to level above
+                    # Do not assume anything for the formation
+                    for i, r in rows.iterrows():
+                        if r["GROUP"] == "UNKNOWN":
+                            rows.loc[i, "GROUP"] = rows.loc[i - 1, "GROUP"]
+
+                    rows_groups = rows.sort_values(["Top_MD", "Base_MD"]).copy()
+                    rows_groups["Level"] = "GROUP"
+                    rows_groups["Lithostrat"] = rows_groups["GROUP"]
+
+                    rows_formations = rows.sort_values(["Top_MD", "Base_MD"]).copy()
+                    rows_formations["Level"] = "FORMATION"
+                    rows_formations["Lithostrat"] = rows_formations["FORMATION"]
+                except Exception:
+                    warnings.warn(
+                        f"No formation tops information was found for {well} in CDF or GIS. Skipping it!"
+                    )
+                    continue
+            else:
+                warnings.warn(
+                    f"No formation tops information was found for {well} in CDF. Skipping it!"
+                )
+                continue
+        else:
+            rows = tops[0].rows(start=None, end=None).to_pandas()
+
+            rows_groups = rows[rows.Level == "GROUP"].sort_values(["Top_MD", "Base_MD"])
+            rows_formations = rows[rows.Level == "FORMATION"].sort_values(
+                ["Top_MD", "Base_MD"]
+            )
 
         group_labels: List[str] = []
         chrono_group_labels: List[str] = []
@@ -789,8 +848,12 @@ def get_vertical_depths(
     """
     vertical_depths_mapper = {}
     for well in well_names:
+        # Trajectory in CDF is fetched from GIS, which agian is fetched from NPD.
+        # NPD do not keep track of technical sidetrackings.
+        well_name = well.split("T")[0].rstrip()
+
         well_data_cdf = client.sequences.list(
-            metadata={"wellbore_name": well, "type": "trajectory"}, limit=None
+            metadata={"wellbore_name": well_name, "type": "trajectory"}, limit=None
         )
         if len(well_data_cdf) == 0:
             warnings.warn(
@@ -1018,89 +1081,6 @@ def get_calibration_values(
     return well_values
 
 
-### VSH HELPERS ###
-
-
-def get_violation_indices(mask: pd.Series) -> pd.DataFrame:
-    """Helper function to retrieve the indices where a mask series is True
-
-    Args:
-        mask (pd.Series): The mask series to retrieve True indices of
-
-    Returns:
-        pd.DataFrame: A dataframe with the columns ["first", "last"] denoting
-            the start and end indices of each block of True values in the
-            passed mask.
-    """
-    counter = (mask != mask.shift(1)).cumsum()
-    indices = (
-        counter.index.to_series().groupby(counter, sort=False).agg(["first", "last"])
-    )
-    values = (
-        mask.groupby(counter).unique().apply(lambda x: x[0] if len(x) == 1 else np.nan)
-    )
-    if values.isna().any():
-        raise ValueError(
-            "More than one unique value found in the one of the mask groups!"
-        )
-
-    return indices.loc[values]
-
-
-def inflection_points(
-    df: pd.DataFrame, curveName: str, before: int, after: int
-) -> Tuple[int, int]:
-    """Helper function for identifying the first inflection point in a curve before and after certain indices.
-
-    Args:
-        df (pd.DataFrame): The dataframe containing the specified curveName.
-        curveName (str): The curve for which to detect inflection points.
-        before (int): The index before which inflection points should be detected
-        after (int): The index after which inflection points should be detected
-
-    Returns:
-        tuple(int, int): The first inflection point in the curve before the before index and after the after index
-            If no inflection point is found, np.nan is returned. If no inflection point before the before index
-            and after the after index is found, a ValueError is raised.
-    """
-    before_df = df.loc[:before, curveName][::-1]
-    after_df = df.loc[after:, curveName]
-    inflection_points = {"before": before_df, "after": after_df}
-    for series_name, series in inflection_points.items():
-        try:
-            with np.errstate(invalid="ignore"):
-                first_derivative = np.gradient(series)
-                if np.isnan(first_derivative).all():
-                    raise IndexError
-                first_inflection_point = (
-                    np.where(
-                        np.diff(
-                            pd.Series(np.sign(first_derivative))
-                            .replace(0, np.nan)
-                            .fillna(method="ffill")
-                        )
-                        != 0
-                    )[0]
-                    + 1
-                )[0]
-                first_inflection_point = series.index[first_inflection_point]
-        except ValueError:
-            # If ValueError, not enough data points to take a gradient, i.e. only
-            # one point in the series. In this case, the first inflection point
-            # is the first and only point in the series.
-            first_inflection_point = series.index[0]
-        except IndexError:
-            # If IndexError, no inflection point found
-            # In both cases return np.nan
-            first_inflection_point = np.nan
-        inflection_points[series_name] = first_inflection_point
-
-    if np.isnan(list(inflection_points.values())).all():
-        raise ValueError("No inflection points found before or after!")
-
-    return inflection_points["before"], inflection_points["after"]
-
-
 def calculate_sampling_rate(array, max_sampling_rate=1):
     """
     Calculates the sampling rate of an array by calculating the weighed
@@ -1123,3 +1103,221 @@ def calculate_sampling_rate(array, max_sampling_rate=1):
     diffs.loc[diffs.index.to_series().gt(max_sampling_rate)] = np.nan
     sampling_rate = (diffs * diffs.index).sum()
     return sampling_rate
+
+
+def squared_euclidean_distance(
+    point_a: npt.NDArray[np.float64], point_b: npt.NDArray[np.float64]
+) -> Any:
+    """
+    Returns the square of the Euclidean distance between two points
+
+    Arguments:
+        point_a (np.ndarray): The first point as [depth, x, y]
+        point_b (np.ndarray): The second point as [depth, x, y]
+
+    Returns:
+        distance (float): The square of the Euclidean distance between point_a
+                          and point b
+    """
+    return np.linalg.norm(point_a - point_b) ** 2
+
+
+def estimate_parameter(
+    depth: List[float],
+    x: List[float],
+    y: List[float],
+    formation: List[str],
+    group: List[str],
+    parameter_train: List[float],
+    depth_train: List[float],
+    x_train: List[float],
+    y_train: List[float],
+    formation_train: List[str],
+    group_train: List[str],
+    distance_function: Callable[
+        [npt.NDArray[np.float64], npt.NDArray[np.float64]], Any
+    ] = squared_euclidean_distance,
+) -> List[float]:
+    """
+    Estimates a parameter at given coordinates and zones based on a calibration list of
+    parameters with corresponding coordinates and zones.
+
+    Arguments:
+        depth (List[float]): List of depths where parameter will be estimated
+        x (List[float]): List of x-coordinates where parameter will be estimated
+        y (List[float]): List of y-coordinates where parameter will be estimated
+        formation (List[str]): List of formations where parameter will be estimated
+        group (List[str]): List of groups where parameter will be estimated
+        parameter_train (List[float]): Calibration parameter values
+        depth_train (List[float]): List of depths for calibration points
+        x_train (List[float]): List of x-coordinates for calibration points
+        y_train (List[float]): List of y-coordinates for calibration points
+        formation_train (List[str]): List of formations for the calibration points
+        group_train (List[str]): List of groups for the calibration points
+        distance_function (Callable([np.ndarray, np.ndarray], float)): Distance function
+                    to use between points in [depth, x, y]
+
+    Return:
+        parameters (List[float]): Estimates of the parameter at the given points
+    """
+    df = pd.DataFrame(
+        {
+            "DEPTH": depth_train,
+            "X": x_train,
+            "Y": y_train,
+            "FORMATION": formation_train,
+            "GROUP": group_train,
+            "PARAMETER": parameter_train,
+        }
+    )
+    parameters = []
+    for i in range(len(x)):
+        # Filter on points in the same formation
+        df_compare = df.loc[df.FORMATION == formation[i]]
+        if df_compare.shape[0] == 0:
+            # If not enough points of matching formation, go for groups
+            df_compare = df.loc[df.GROUP == group[i]]
+            if df_compare.shape[0] == 0:
+                # If not enough points of matching group or formation, use all points
+                df_compare = df
+        # Compute the distance between the point and all points in
+        # the dataframe of comparable calibration points
+        point = np.array([depth[i], x[i], y[i]])
+        calibration_points = df_compare[["DEPTH", "X", "Y"]].to_numpy()
+        calibration_points_parameters = df_compare.PARAMETER.values
+        distances = np.apply_along_axis(distance_function, 1, calibration_points, point)
+        # Remove any points that happen to be at zero distance
+        zeros_indexes = [i for i, x in enumerate(distances) if x == 0.0]
+        for ind in sorted(zeros_indexes, reverse=True):
+            distances = np.delete(distances, ind)
+            calibration_points_parameters = np.delete(
+                calibration_points_parameters, ind
+            )
+        # Compute weighted average based on distance
+        parameters.append(
+            np.sum(1.0 / distances * calibration_points_parameters)
+            / np.sum(1.0 / distances)
+        )
+    return parameters
+
+
+def get_cognite_client(
+    cognite_api_key: str = None,
+    cognite_project_id: str = "akbp-subsurface",
+    cognite_client_id: str = "mlpet",
+) -> CogniteClient:
+    """
+    Helper function to create a CogniteClient instance.
+
+    Args:
+        cognite_api_key (str): The API key to use for the CogniteClient
+        cognite_project_id (int): The project ID to use for the CogniteClient
+        cognite_client_id (str): The client ID to use for the CogniteClient
+
+        Returns:
+            CogniteClient: The created CogniteClient instance
+    """
+    if cognite_api_key is None:
+        try:
+            cognite_api_key = os.environ["COGNITE_API_KEY"]
+        except KeyError as e:
+            raise ValueError(
+                "No API key provided and no environment variable "
+                "COGNITE_API_KEY found!"
+            ) from e
+    return CogniteClient(
+        api_key=cognite_api_key,
+        project=cognite_project_id,
+        client_name=cognite_client_id,
+    )
+
+
+def run_CDF_function(
+    df: pd.DataFrame,
+    id_column: str,
+    keyword_arguments: Dict[str, Any],
+    external_id: str,
+    client: CogniteClient = None,
+    return_file: bool = True,
+) -> pd.DataFrame:
+    """
+    Helper function to run a CDF function.
+
+        Args:
+            df (pd.DataFrame): The dataframe to run the function on
+            id_column (str): The column to use as the ID for the function
+            keyword_arguments (Dict[str, Any]): The keyword arguments to use for the function
+            external_id (str): The external ID to use for the function
+            client (CogniteClient): The CogniteClient instance to use for the function
+            return_file (bool): Whether to return the predictions as a CDF file link.
+                Defaults to True.
+
+        Returns:
+            pd.DataFrame: The dataframe with the results of the function
+    """
+    # Create the CogniteClient instance
+    if client is None:
+        client = get_cognite_client()
+    function = client.functions.retrieve(external_id=external_id)
+    if function is None:
+        raise ValueError(
+            f"No function with external ID {external_id} found in Cognite!"
+        )
+    required_input = eval(function.to_pandas().loc["required_input", "value"])
+    optional_input = eval(function.to_pandas().loc["optional_input", "value"])
+    if not np.all([c in df.columns for c in required_input]):
+        raise ValueError(
+            f"The dataframe provided does not contain all required input"
+            f"for the function ({required_input})"
+        )
+    input_curves = [c for c in required_input + optional_input if c in df.columns]
+    # We check that the input curves to the function will not have nans before sending them to the function
+    df_ = df.copy()
+    df_ = preprocessors.fillna_with_fillers(
+        df_,
+        num_filler=keyword_arguments["nan_numerical_value"],
+        cat_filler=keyword_arguments["nan_textual_value"],
+    )
+    # Function call: prepare data as function payload description and fetch results
+    # Ensure no NaNs in id_column
+    if df[id_column].isna().any():
+        raise ValueError(
+            f"The id_column {id_column} contains NaNs, which is not allowed!"
+        )
+    in_data = {
+        "input": [
+            {
+                "well": k,
+                "input_logs": v[input_curves].to_dict(orient="list"),
+                "keyword_arguments": keyword_arguments,
+            }
+            for k, v in df_.groupby(id_column)
+        ],
+        "return_file": return_file,
+    }
+    results = function.call(in_data).get_response()
+    if results["status"] == "error":
+        raise ValueError(f"Function returned an error: {results['error message']}")
+    # get function results to df
+    output_curves = eval(function.to_pandas().loc["output_curves", "value"])
+    df_[output_curves] = np.nan
+    if return_file:
+        predictions = json.loads(
+            client.files.download_bytes(external_id=results["prediction_file"])
+        )
+    else:
+        predictions = results["prediction"]
+    for well in predictions:
+        pred = pd.DataFrame.from_dict(well)
+        df_.loc[df_[id_column] == well["well"], output_curves] = pred[
+            output_curves
+        ].values
+
+    # Fill imputed values back to original
+    df_ = preprocessors.set_as_nan(
+        df_,
+        numerical_value=keyword_arguments["nan_numerical_value"],
+        categorical_value=keyword_arguments["nan_textual_value"],
+    )
+
+    return df_
